import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import { useAppSelector } from '@/redux/hooks/useAppSelector';
import { activePluginsDataSelector } from '@/redux/plugins/plugins.selectors';
import { setCurrentDrawerPluginHash } from '@/redux/plugins/plugins.slice';
import { PluginsManager } from '@/services/pluginsManager';
import { showToast } from '@/utils/showToast';
import axios from 'axios';
import { ChangeEvent, useMemo, useState } from 'react';
import { getErrorMessage } from '@/utils/getErrorMessage';
import { PLUGIN_LOADING_ERROR_PREFIX } from '../../AvailablePluginsDrawer.constants';

type UseLoadPluginReturnType = {
  handleChangePluginUrl: (event: ChangeEvent<HTMLInputElement>) => void;
  handleLoadPlugin: () => Promise<void>;
  isPending: boolean;
  pluginUrl: string;
};

export const useLoadPluginFromUrl = (): UseLoadPluginReturnType => {
  const [pluginUrl, setPluginUrl] = useState('');
  const [isLoading, setIsLoading] = useState(false);
  const activePlugins = useAppSelector(activePluginsDataSelector);
  const dispatch = useAppDispatch();

  const isPending = useMemo(
    () => !pluginUrl || isLoading || !URL.canParse(pluginUrl),
    [pluginUrl, isLoading],
  );

  const handleSetCurrentDrawerPluginHash = (hash: string): void => {
    dispatch(setCurrentDrawerPluginHash(hash));
  };

  const handleLoadPlugin = async (): Promise<void> => {
    try {
      setIsLoading(true);
      const response = await axios(pluginUrl);
      let pluginScript = response.data;

      const hash = PluginsManager.setHashedPlugin({
        pluginUrl,
        pluginScript,
      });

      pluginScript += `//# sourceURL=${pluginUrl}`;

      /* eslint-disable no-new-func */
      const loadPlugin = new Function(pluginScript);

      if (!(hash in activePlugins)) {
        loadPlugin();
      }

      setPluginUrl('');

      handleSetCurrentDrawerPluginHash(hash);
    } catch (error) {
      const errorMessage = getErrorMessage({ error, prefix: PLUGIN_LOADING_ERROR_PREFIX });
      showToast({
        type: 'error',
        message: errorMessage,
      });
    } finally {
      setIsLoading(false);
    }
  };
  const handleChangePluginUrl = (event: ChangeEvent<HTMLInputElement>): void => {
    setPluginUrl(event.target.value);
  };

  return {
    handleChangePluginUrl,
    handleLoadPlugin,
    isPending,
    pluginUrl,
  };
};
